/*
 * Asterisk -- An open source telephony toolkit.
 *
 * Copyright (c) 2008 Roberto Casas.  All rights reserved.
 *
 * Roberto Casas <roberto.casas@diaple.com>
 *
 * This code is released by the author with no restrictions on usage.
 *
 * See http://www.asterisk.org for more information about
 * the Asterisk project. Please do not directly contact
 * any of the maintainers of this project for assistance;
 * the project provides a web site, mailing lists and IRC
 * channels for your use.
 *
 */

/*! \file
 *
 * \brief Originate application
 *
 * \author Roberto Casas <roberto.casas@diaple.com>
 * 
 * \ingroup applications
 */

#include "asterisk.h"

ASTERISK_FILE_VERSION(__FILE__, "$Revision: $")

#include "asterisk/file.h"
#include "asterisk/channel.h"
#include "asterisk/pbx.h"
#include "asterisk/module.h"
#include "asterisk/app.h"

/*! The timeout for originated calls, in seconds */
#define TIMEOUT 30

static char *app_originate = "Originate";

static char *originate_synopsis = "Originates a call like the Originate for CLI or AMI";

static char *originate_descrip =
"Originate(tech_data,type,arg1,arg2)\n"
"  tech_data - Technology and data to construct the originating channel.\n"
"              By example SIP/1234.\n"
"  type      - 'app' or 'exten' depending if we want to bridge the tech_data to an extension\n"
"              or to an application.\n"
"  arg1       - If 'app' type: the application name\n"
"               If 'exten' type: the destination context\n"
"  arg2       - If 'app' type: the application data all arguments from here will be passed to the application\n"
"               If 'exten' type: the destination extension inside the specified context\n";

static int originate_exec(struct ast_channel *chan, void *data)
{
	AST_DECLARE_APP_ARGS(args,
		AST_APP_ARG(tech_data);
		AST_APP_ARG(type);
		AST_APP_ARG(arg1);
                AST_APP_ARG(arg2);
	);
	char *parse;
        char *chantech;
        char *chandata;
        int res = 0;
	
	if (!data)
		return 0;

        if (args.argc != 4) {
                ast_log(LOG_ERROR, "Incorrect number of arguments\n");
                return -1;
	}

	parse = ast_strdupa(data);

	AST_STANDARD_APP_ARGS(args, parse);

	ast_autoservice_start(chan);
	
	chandata = ast_strdupa(args.tech_data);
        chantech = strsep(&chandata, "/");
	if (!chandata) {
		ast_log(LOG_ERROR, "No data provided after channel type: %s\n", args.tech_data);
                return -1;
        }

	if(strcmp(args.type, "exten") == 0) {
		ast_pbx_outgoing_exten(chantech, AST_FORMAT_SLINEAR, chandata, TIMEOUT * 1000, args.arg1, args.arg2, 1, &res, 0, NULL, NULL, NULL, NULL, NULL);
	} else if(strcmp(args.type, "app") == 0) {
		ast_pbx_outgoing_app(chantech, AST_FORMAT_SLINEAR, chandata, TIMEOUT * 1000, args.arg1, args.arg2, &res, 0, NULL, NULL, NULL, NULL, NULL);
	} else {
		ast_log(LOG_ERROR, "Incorrect type, it should be 'exten' or 'app': %s\n", args.type);
                return -1;
	}

	ast_autoservice_stop(chan);

	return res;
}

static int unload_module(void)
{
	int res;
	
	res = ast_unregister_application(app_originate);
	
	return res;
}

static int load_module(void)
{
	int res;
	
	res = ast_register_application(app_originate, originate_exec, originate_synopsis, originate_descrip);
	
	return res;
}

AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "Originate call");
